#objdump: -rs
#name: x86-64 imgrel (RVA) reloc

.*: +file format pe-x86-64

RELOCATION RECORDS FOR \[\.text\]:
OFFSET[ 	]+TYPE[ 	]+VALUE
0+02 IMAGE_REL_AMD64_SECTION  \.text
0+07 IMAGE_REL_AMD64_SECTION  \.data
0+0c IMAGE_REL_AMD64_SECTION  \.text
0+11 IMAGE_REL_AMD64_SECTION  Xtrn

RELOCATION RECORDS FOR \[\.data\]:
OFFSET[ 	]+TYPE[ 	]+VALUE
0+00 IMAGE_REL_AMD64_SECTION  \.text
0+02 IMAGE_REL_AMD64_SECTION  \.data
0+04 IMAGE_REL_AMD64_SECTION  \.data
0+06 IMAGE_REL_AMD64_SECTION  Xtrn

Contents of section \.text:
 0000 66b90000 6681c100 006681e9 09006681  .*
 0010 f10000.*

Contents of section \.data:
 0000 00000000 04000000 00000000 00000000  .*
